@@ -8,6 +8,7 @@
 #pragma once
 
 #include <memory>
+#include <stdint.h>
 #include <string>
 #include <variant>
 #include <vector>
@@ -62,6 +63,11 @@ public:
 				       Transform transform = Transform::Identity,
 				       V4L2SubdeviceFormat *sensorFormat = nullptr) = 0;
 
+	virtual V4L2Subdevice::Stream imageStream() const;
+	virtual std::optional<V4L2Subdevice::Stream> embeddedDataStream() const;
+	virtual V4L2SubdeviceFormat embeddedDataFormat() const;
+	virtual int setEmbeddedDataEnabled(bool enable);
+
 	virtual const ControlList &properties() const = 0;
 	virtual int sensorInfo(IPACameraSensorInfo *info) const = 0;
 	virtual Transform computeTransform(Orientation *orientation) const = 0;
