@@ -131,8 +131,6 @@ private:
 	void setSensorControls(const ControlList &sensorControls);
 
 	std::string id_;
-	std::vector<unsigned int> tpgCodes_;
-	std::vector<Size> tpgSizes_;
 	Size tpgResolution_;
 };
 
@@ -159,15 +157,16 @@ int MaliC55CameraData::init()
 	 */
 	sensor_ = CameraSensorFactoryBase::create(entity_);
 	if (!sensor_)
-		return ret;
+		return -ENODEV;
 
 	const MediaPad *sourcePad = entity_->getPadByIndex(0);
 	MediaEntity *csiEntity = sourcePad->links()[0]->sink()->entity();
 
 	csi_ = std::make_unique<V4L2Subdevice>(csiEntity);
-	if (csi_->open()) {
+	ret = csi_->open();
+	if (ret) {
 		LOG(MaliC55, Error) << "Failed to open CSI-2 subdevice";
-		return false;
+		return ret;
 	}
 
 	return 0;
@@ -180,16 +179,15 @@ void MaliC55CameraData::initTPGData()
 	if (formats.empty())
 		return;
 
-	tpgCodes_ = utils::map_keys(formats);
-	std::sort(tpgCodes_.begin(), tpgCodes_.end());
+	std::vector<Size> tpgSizes;
 
 	for (const auto &format : formats) {
 		const std::vector<SizeRange> &ranges = format.second;
-		std::transform(ranges.begin(), ranges.end(), std::back_inserter(tpgSizes_),
+		std::transform(ranges.begin(), ranges.end(), std::back_inserter(tpgSizes),
 			       [](const SizeRange &range) { return range.max; });
 	}
 
-	tpgResolution_ = tpgSizes_.back();
+	tpgResolution_ = tpgSizes.back();
 }
 
 void MaliC55CameraData::setSensorControls(const ControlList &sensorControls)
@@ -620,7 +618,7 @@ public:
 	void imageBufferReady(FrameBuffer *buffer);
 	void paramsBufferReady(FrameBuffer *buffer);
 	void statsBufferReady(FrameBuffer *buffer);
-	void paramsComputed(unsigned int requestId);
+	void paramsComputed(unsigned int requestId, uint32_t bytesused);
 	void statsProcessed(unsigned int requestId, const ControlList &metadata);
 
 	bool match(DeviceEnumerator *enumerator) override;
@@ -1496,7 +1494,7 @@ void PipelineHandlerMaliC55::statsBufferReady(FrameBuffer *buffer)
 				 sensorControls);
 }
 
-void PipelineHandlerMaliC55::paramsComputed(unsigned int requestId)
+void PipelineHandlerMaliC55::paramsComputed(unsigned int requestId, uint32_t bytesused)
 {
 	MaliC55FrameInfo &frameInfo = frameInfoMap_[requestId];
 	Request *request = frameInfo.request;
@@ -1507,8 +1505,7 @@ void PipelineHandlerMaliC55::paramsComputed(unsigned int requestId)
 	 * video devices.
 	 */
 
-	frameInfo.paramBuffer->_d()->metadata().planes()[0].bytesused =
-		sizeof(struct mali_c55_params_buffer);
+	frameInfo.paramBuffer->_d()->metadata().planes()[0].bytesused = bytesused;
 	params_->queueBuffer(frameInfo.paramBuffer);
 	stats_->queueBuffer(frameInfo.statBuffer);
 
@@ -1712,7 +1709,7 @@ bool PipelineHandlerMaliC55::match(DeviceEnumerator *enumerator)
 	 *
 	 * MEDIA_ENT_F_CAM_SENSOR - The test pattern generator
 	 * MEDIA_ENT_F_VID_IF_BRIDGE - A CSI-2 receiver
-	 * MEDIA_ENT_F_IO_V4L - An input device
+	 * MEDIA_ENT_F_PROC_VIDEO_PIXEL_FORMATTER - An input device
 	 *
 	 * The last one will be unsupported for now. The TPG is relatively easy,
 	 * we just register a Camera for it. If we have a CSI-2 receiver we need
@@ -1738,7 +1735,7 @@ bool PipelineHandlerMaliC55::match(DeviceEnumerator *enumerator)
 				return registered;
 
 			break;
-		case MEDIA_ENT_F_IO_V4L:
+		case MEDIA_ENT_F_PROC_VIDEO_PIXEL_FORMATTER:
 			LOG(MaliC55, Warning) << "Memory input not yet supported";
 			break;
 		default:
